// -*- C++ -*-
/*!
 * @file SharedMemory_posix.cpp
 * @brief SharedMemory class
 * @date $Date$
 * @author Nobuhiko Miyamoto <n-miyamoto@aist.go.jp>
 *
 * Copyright (C) 2017 Nobuhiko Miyamoto
 *     Robot Innovation Research Center,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id$
 *
 */


#include <coil/SharedMemory.h>
//#include "SharedMemory.h"
#include <string.h>


namespace coil
{
  /*!
   * @if jp
   * @brief 󥹥ȥ饯
   * @else
   * @brief Constructor
   * @endif
   */
  SharedMemory::SharedMemory()
    : m_memory_size(0),
      m_shm(NULL),
      m_file_create(false),
      m_fd(-1)
  {
  }


  /*!
   * @if jp
   * @brief ǥȥ饯
   * @else
   * @brief Destructor
   * @endif
   */
  SharedMemory::~SharedMemory()
  {
    close();
  }

  /*!
   * @if jp
   * @brief ԡ󥹥ȥ饯
   * @else
   * @brief Copy Constructor
   * @endif
   */
  SharedMemory::SharedMemory(const SharedMemory& rhs)
  {
    m_memory_size = rhs.m_memory_size;
    m_shm_address = rhs.m_shm_address;
    m_shm = rhs.m_shm;
    m_fd = rhs.m_fd;
 

  }

  /*!
   * @if jp
   * @brief 黻
   * @else
   * @brief Assignment operator
   * @endif
   */
  SharedMemory& SharedMemory::operator=(const SharedMemory& rhs)
  {
    SharedMemory tmp(rhs);
    std::swap(this->m_memory_size, tmp.m_memory_size);
    std::swap(this->m_shm_address, tmp.m_shm_address);
    std::swap(this->m_shm, tmp.m_shm);
    std::swap(this->m_fd, tmp.m_fd);
    std::swap(this->m_file_create, tmp.m_file_create);

    return *this;
  }

  /*!
   * @if jp
   *
   * @brief ͭ
   *
   *
   * @param shm_address ͭμ̻
   * @param memory_size ͭΥ
   *
   * @return 0: , -1: 
   *
   * @else
   *
   * @brief Create Shared Memory 
   *
   *
   * @param shm_address 
   * @param memory_size 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::create(std::string shm_address,
                     int memory_size)
  {

    m_shm_address = shm_address;
    m_memory_size = memory_size;
 

    m_fd = shm_open(m_shm_address.c_str(), O_RDWR|O_CREAT, S_IRUSR|S_IWUSR|S_IRGRP|S_IWGRP|S_IROTH);
    if(m_fd < 0)
    {
        return -1;
    }
    ftruncate(m_fd, m_memory_size);
    m_shm = (char*)mmap(NULL,
            m_memory_size,
            PROT_READ|PROT_WRITE,
            MAP_SHARED,
            m_fd,
            0);

    m_file_create = true;
    return 0;
  }

  /*!
   * @if jp
   *
   * @brief ͭؤΥ
   *
   *
   * @param shm_address ͭμ̻
   *
   * @return 0: , -1: 
   *
   * @else
   *
   * @brief Open Shared Memory 
   *
   *
   * @param shm_address 
   * @param memory_size 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::open(std::string shm_address, int memory_size)
  {
    m_shm_address = shm_address;
	m_memory_size = memory_size;


    m_fd = shm_open(m_shm_address.c_str(), O_RDWR|O_CREAT, 0);
    if(m_fd < 0)
    {
        return -1;
    }
    ftruncate(m_fd, m_memory_size);
    m_shm = (char*)mmap(NULL,
            m_memory_size,
            PROT_READ|PROT_WRITE,
            MAP_SHARED,
            m_fd,
            0);
 
    return 0;
  }

  /*!
   * @if jp
   *
   * @brief ͭؤν񤭹
   *
   *
   * @param data 񤭹ǡ
   *
   * @return 0: , -1: 
   *
   * @else
   *
   * @brief Write Shared Memory 
   *
   *
   * @param data 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::write(const char *data, const int pos, const int size)
  {
	  if (!created())
	  {
		  return -1;
	  }

	  memcpy(&m_shm[pos],&data[0],size);
    
	  return 0;
  }

  /*!
   * @if jp
   *
   * @brief ꤫ͭɤ߹
   *
   *
   * @param data ɤ߹ǡ
   *
   * @return 0: , -1: 
   *
   * @else
   *
   * @brief Read Shared Memory 
   *
   *
   * @param data 
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::read(char* data, const int pos, const int size)
  {
	  if (!created())
	  {
		  return -1;
	  }

	  memcpy(&data[0],&m_shm[pos],size);

	  return 0;
  }

  /*!
   * @if jp
   *
   * @brief ͭκ
   *
   *
   *
   * @return 0: , -1: 
   *
   * @else
   *
   * @brief Close Shared Memory 
   *
   *
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::close()
  {
    
    if (created())
    {
	::close(m_fd);
    }
    else
    {
	return -1;
    }
    if(m_file_create)
    {
	//shm_unlink(m_shm_address);
    }
    return 0;

  }
  /*!
   * @if jp
   *
   * @brief Υ
   *
   *
   *
   * @return 
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return size
   *
   * @endif
   */
  unsigned int SharedMemory::get_size()
  {
	return m_memory_size;
  }
  /*!
   * @if jp
   *
   * @brief Υɥ쥹
   *
   *
   *
   * @return ɥ쥹
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return address
   *
   * @endif
   */
  std::string SharedMemory::get_addresss()
  {
	return m_shm_address;
  };
  /*!
   * @if jp
   *
   * @brief μ
   *
   *
   *
   * @return 
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return memory
   *
   * @endif
   */
  char *SharedMemory::get_data()
  {
	return m_shm;
  }



  /*!
   * @if jp
   *
   * @brief ե(Windowsξϲ⤷ʤ)
   *
   *
   *
   * @return 0: , -1: 
   *
   * @else
   *
   * @brief 
   *
   *
   *
   * @return 0: successful, -1: failed
   *
   * @endif
   */
  int SharedMemory::unlink()
  {
	shm_unlink(m_shm_address.c_str());
	return 0;
  }


  /*!
  * @if jp
  *
  * @brief ͭѤߤγǧ
  *
  *
  *
  * @return true: Ѥ, false: ̤
  *
  * @else
  *
  * @brief 
  *
  *
  *
  * @return
  *
  * @endif
  */
  bool SharedMemory::created()
  {
	if(m_fd < 0)
	{
		return false;
	}
	else
	{
		return true;
	}
  }




}
